<!--
  Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
-->

<!-- Translations: Tagalog -->
{% macro t(key) %}{{ {
  "language": "tl",
  "action.edit": "I-edit ang pahinang ito",
  "action.skip": "I-skip tungo sa nilalaman",
  "action.view": "Tingnan ang pinagmulan ng pahinang ito",
  "announce.dismiss": "Huwag mo nang ipakita muli ito",
  "blog.categories": "Kategorya",
  "blog.categories.in": "sa",
  "blog.continue": "Magpatuloy sa pagbabasa",
  "blog.index": "Bumalik sa index",
  "blog.references": "Mga kaugnay na link",
  "clipboard.copy": "Kopyahin sa clipboard",
  "clipboard.copied": "Nakopya mula sa clipboard",
  "consent.accept": "Tanggapin",
  "consent.manage": "Pamahalaan ang mga setting",
  "consent.reject": "Tanggihan",
  "footer": "Lagdang Pangwakas",
  "footer.next": "Susunod",
  "footer.previous": "Nakaraan",
  "header": "Pamuhatan",
  "meta.comments": "Mga Komento",
  "meta.source": "Pinagmulan",
  "nav": "Nabigasyon",
  "readtime.one": "1 minutong basahin",
  "readtime.other": "# minutong basahin",
  "rss.created": "RSS feed",
  "rss.updated": "RSS feed ng na-update na nilalaman",
  "search": "Hanapin",
  "search.placeholder": "Hanapin",
  "search.share": "Ibahagi",
  "search.reset": "Tanggalin",
  "search.result.initializer": "Sinisimulan ang paghahanap",
  "search.result.placeholder": "Mag-type upang simulan ang paghahanap",
  "search.result.none": "Walang nahanap na dokumento",
  "search.result.one": "1 magkatugmang dokumento",
  "search.result.other": "# magkatugmang mga dokumento",
  "search.result.more.one": "1 meron sa pahina na ito",
  "search.result.more.other": "# meron sa pahina na ito",
  "search.result.term.missing": "Nawawala",
  "select.language": "Pumili ng lenguwahe",
  "select.version": "Pumili ng bersyon",
  "source": "Pumunta sa repository",
  "source.file.contributors": "Mga nag-ambag",
  "source.file.date.created": "Nagawa",
  "source.file.date.updated": "Huling update",
  "tabs": "Mga tala",
  "toc": "Talaan ng nilalaman",
  "top": "Bumalik sa taas"
}[key] }}{% endmacro %}
