////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Dialog
.md-dialog {
  position: fixed;
  inset-inline-end: px2rem(16px);
  bottom: px2rem(16px);
  z-index: 4;
  min-width: px2rem(222px);
  padding: px2rem(8px) px2rem(12px);
  pointer-events: none;
  background-color: var(--md-default-fg-color);
  border-radius: px2rem(2px);
  box-shadow: var(--md-shadow-z3);
  opacity: 0;
  transition:
    transform 0ms   400ms,
    opacity   400ms;
  transform: translateY(100%);

  // [print]: Hide dialog
  @media print {
    display: none;
  }

  // Active dialog
  &--active {
    pointer-events: initial;
    opacity: 1;
    transition:
      transform 400ms cubic-bezier(0.075, 0.85, 0.175, 1),
      opacity   400ms;
    transform: translateY(0);
  }

  // Dialog wrapper
  &__inner {
    font-size: px2rem(14px);
    color: var(--md-default-bg-color);
  }
}
