////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Author, i.e., GitHub user
  .md-author {
    position: relative;
    display: block;
    flex-shrink: 0;
    width: px2rem(32px);
    height: px2rem(32px);
    overflow: hidden;
    border-radius: 100%;
    transition:
      color 125ms,
      transform 125ms;

    // Author image
    img {
      display: block;
    }

    // More authors
    &--more {
      font-size: px2rem(12px);
      font-weight: 700;
      line-height: px2rem(32px);
      color: var(--md-default-fg-color--lighter);
      text-align: center;
      background: var(--md-default-fg-color--lightest);
    }

    // Enlarge image
    &--long {
      width: px2rem(48px);
      height: px2rem(48px);
    }
  }

  // Author link
  a.md-author {
    transform: scale(1);

    // Author image
    img {
      filter: grayscale(100%) opacity(75%);
      // Hack: also apply this here, in order to mitigate browser glitches in
      // Chrome and Edge when hovering the avatar - see https://t.ly/Q3ECC
      border-radius: 100%;
      transition: filter 125ms;
    }

    // Author on focus/hover
    &:is(:focus, :hover) {
      z-index: 1;
      transform: scale(1.1);

      // Author image
      img {
        filter: grayscale(0%);
      }
    }
  }
}
