////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Status variables
:root {
  --md-status: svg-load("material/information-outline.svg");
  --md-status--new: svg-load("material/alert-decagram.svg");
  --md-status--deprecated: svg-load("material/trash-can.svg");
  --md-status--encrypted: svg-load("material/shield-lock.svg");
}

// ----------------------------------------------------------------------------

// Status
.md-status {

  // Status icon
  &::after {
    display: inline-block;
    width: px2em(18px);
    height: px2em(18px);
    vertical-align: text-bottom;
    content: "";
    background-color: var(--md-default-fg-color--light);
    mask-image: var(--md-status);
    mask-repeat: no-repeat;
    mask-position: center;
    mask-size: contain;
  }

  // Status icon on hover
  &:hover::after {
    background-color: currentcolor;
  }

  // Status: new
  &--new::after {
    mask-image: var(--md-status--new);
  }

  // Status: deprecated
  &--deprecated::after {
    mask-image: var(--md-status--deprecated);
  }

  // Status: encrypted
  &--encrypted::after {
    mask-image: var(--md-status--encrypted);
  }
}
