////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Metadata
.md-meta {
  font-size: px2rem(14px);
  line-height: 1.3;
  color: var(--md-default-fg-color--light);

  // Metadata list
  &__list {
    display: inline-flex;
    flex-wrap: wrap;
    padding: 0;
    margin: 0;
    list-style: none;
  }

  // Metadata item separator
  &__item:not(:last-child)::after {
    margin-inline: px2rem(4px);
    content: "·";
  }

  // Metadata link
  &__link {
    color: var(--md-typeset-a-color);

    // Metadata link on focus/hover
    &:is(:focus, :hover) {
      color: var(--md-accent-fg-color);
    }
  }
}

// Draft
.md-draft {
  display: inline-block;
  padding-inline: px2em(8px, 14px);
  font-weight: 700;
  color: hsla(255, 100%, 100%);
  background-color: $clr-red-a400;
  border-radius: px2em(2px);
}
