////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Banner for announcements and warnings
.md-banner {
  color: var(--md-footer-fg-color--lighter);

  // Don't wrap name of blog article
  strong {
    color: var(--md-footer-fg-color);
    white-space: nowrap;
  }

  a {
    color: var(--md-footer-fg-color);

    &:focus,
    &:hover {
      color: currentcolor;

      .twemoji {
        background-color: var(--md-footer-fg-color);
        box-shadow: none;
      }
    }
  }

  .twemoji {
    display: inline-block;
    width: px2rem(24px);
    height: px2rem(24px);
    padding: px2rem(5px);
    vertical-align: bottom;
    border-radius: 100%;
    box-shadow: 0 0 0 px2rem(1px) currentcolor inset;
    transition: all 250ms;

    svg {
      display: block;
      max-height: initial;
    }
  }
}
