<!--
  Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
-->

<!-- Translations: Japanese -->
{% macro t(key) %}{{ {
  "language": "ja",
  "action.edit": "編集",
  "action.skip": "コンテンツにスキップ",
  "action.view": "このページの原文を表示",
  "announce.dismiss": "非表示にします",
  "blog.archive": "過去の投稿",
  "blog.categories": "カテゴリー",
  "blog.categories.in": "",
  "blog.continue": "続きを読む",
  "blog.draft": "下書き",
  "blog.index": "ブログトップへ戻る",
  "blog.meta": "メタデータ",
  "blog.references": "関連リンク",
  "clipboard.copy": "クリップボードへコピー",
  "clipboard.copied": "コピーしました",
  "consent.accept": "同意",
  "consent.manage": "サイトの設定",
  "consent.reject": "拒否",
  "footer": "フッター",
  "footer.next": "次",
  "footer.previous": "前",
  "header": "ヘッダー",
  "meta.comments": "コメント",
  "meta.source": "ソース",
  "nav": "ナビゲーション",
  "readtime.one": "このページは約1分で読めます",
  "readtime.other": "このページは約#分で読めます",
  "rss.created": "新しいページのRSSフィード",
  "rss.updated": "更新されたページのRSSフィード",
  "search": "検索",
  "search.config.lang": "ja",
  "search.config.pipeline": "stemmer",
  "search.config.separator": "[\\s\\-　、。，．]+",
  "search.placeholder": "検索",
  "search.share": "共有",
  "search.reset": "クリア",
  "search.result.initializer": "検索を初期化",
  "search.result.placeholder": "検索キーワードを入力してください",
  "search.result.none": "何も見つかりませんでした",
  "search.result.one": "1件見つかりました",
  "search.result.other": "#件見つかりました",
  "search.result.more.one": "このページ内にもう1件見つかりました",
  "search.result.more.other": "このページ内にあと#件見つかりました",
  "search.result.term.missing": "検索に含まれない",
  "select.language": "言語切り替え",
  "select.version": "バージョン切り替え",
  "source": "リポジトリへ",
  "source.file.contributors": "投稿者",
  "source.file.date.created": "作成日",
  "source.file.date.updated": "最終更新日",
  "tabs": "タブ",
  "toc": "目次",
  "top": "ページトップへ戻る"
}[key] }}{% endmacro %}
