////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Emoji and icon container
  :is(.emojione, .twemoji, .gemoji) {
    --md-icon-size: #{px2em(18px)};

    display: inline-flex;
    height: var(--md-icon-size);
    vertical-align: text-top;

    // Icon - inlined via mkdocs-material-extensions
    svg {
      width: var(--md-icon-size);
      max-height: 100%;
      fill: currentcolor;
    }
  }

  // Icon with size modifier
  :is(.lg, .xl, .xxl, .xxxl) {
    vertical-align: text-bottom;
  }

  // Adjust icon alignment
  .middle {
    vertical-align: middle;
  }

  // Adjust icon size to 1.5x
  .lg {
    --md-icon-size: #{px2em(24px)};
  }

  // Adjust icon size to 2x
  .xl {
    --md-icon-size: #{px2em(36px)};
  }

  // Adjust icon size to 3x
  .xxl {
    --md-icon-size: #{px2em(48px)};
  }

  // Adjust icon size to 4x
  .xxxl {
    --md-icon-size: #{px2em(64px)};
  }
}
