////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Arithmatex container
  div.arithmatex {
    overflow: auto;

    // [mobile -]: Align with body copy
    @include break-to-device(mobile) {
      margin: 0 px2rem(-16px);

      // Arithmatex content
      > * {
        width: min-content;
      }
    }

    // Arithmatex content
    > * {
      padding: 0 px2rem(16px);
      margin-inline: auto !important; // stylelint-disable-line
      touch-action: auto;

      // MathJax container - see https://bit.ly/3HR8YJ5
      mjx-container {
        margin: 0 !important; // stylelint-disable-line
      }
    }

    // Prevent horizontal overflow, as this element is not visible but still has
    // a height, which might be a bug in MathJax - see https://t.ly/ckPiA
    mjx-assistive-mml {
      height: 0;
    }
  }
}
