////
/// Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Pagination
.md-pagination {
  display: flex;
  gap: px2rem(8px);
  align-items: center;
  justify-content: center;
  font-size: px2rem(16px);
  font-weight: 700;

  // Pagination item
  > * {
    display: flex;
    align-items: center;
    justify-content: center;
    min-width: px2rem(36px);
    height: px2rem(36px);
    text-align: center;
    border-radius: px2rem(4px);
  }

  // Active pagination item
  &__current {
    color: var(--md-default-fg-color--light);
    background-color: var(--md-default-fg-color--lightest);
  }

  // Pagination link
  &__link {
    transition:
      color            125ms,
      background-color 125ms;

    // Pagination link on focus/hover
    &:is(:focus, :hover) {
      color: var(--md-accent-fg-color);
      background-color: var(--md-accent-fg-color--transparent);

      // Pagination icon
      svg {
        color: var(--md-accent-fg-color);
      }
    }

    // Show outline for keyboard devices
    &.focus-visible {
      outline-color: var(--md-accent-fg-color);
      outline-offset: px2rem(4px);
    }

    // Pagination icon
    svg {
      display: block;
      width: px2rem(24px);
      max-height: 100%;
      color: var(--md-default-fg-color--lighter);
      fill: currentcolor;
    }
  }
}
