<!--
  Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
-->

<!-- Translations: Danish -->
{% macro t(key) %}{{ {
  "language": "da",
  "action.edit": "Redigér denne side",
  "action.skip": "Gå til indholdet",
  "action.view": "Vis kildetekst på denne side",
  "announce.dismiss": "Vis ikke dette igen",
  "blog.archive": "Arkiv",
  "blog.categories": "Kategorier",
  "blog.categories.in": "i",
  "blog.continue": "Læs mere",
  "blog.draft": "Udkast",
  "blog.index": "Gå tilbage",
  "blog.meta": "Metadata",
  "blog.references": "Relateret indhold",
  "clipboard.copy": "Kopiér til udklipsholderen",
  "clipboard.copied": "Kopieret til udklipsholderen",
  "consent.accept": "Accepter",
  "consent.manage": "Administrer indstillinger",
  "consent.reject": "Afvis",
  "footer": "Sidefod",
  "footer.next": "Næste",
  "footer.previous": "Forrige",
  "header": "Sidehoved",
  "meta.comments": "Kommentarer",
  "meta.source": "Kilde",
  "nav": "Navigation",
  "readtime.one": "1 minuts læsetid",
  "readtime.other": "# minutters læsetid",
  "rss.created": "RSS feed",
  "rss.updated": "RSS feed af opdateret indhold",
  "search": "Søg",
  "search.config.lang": "da",
  "search.placeholder": "Søg",
  "search.share": "Del",
  "search.reset": "Nulstil søgning",
  "search.result.initializer": "Start søgning",
  "search.result.placeholder": "Indtast søgeord",
  "search.result.none": "Ingen resultater fundet",
  "search.result.one": "1 resultat",
  "search.result.other": "# resultater",
  "search.result.more.one": "1 resultat mere på denne side",
  "search.result.more.other": "# resultater mere på denne side",
  "search.result.term.missing": "Mangler",
  "select.language": "Vælg sprog",
  "select.version": "Vælg version",
  "source": "Åbn arkiv",
  "source.file.contributors": "Bidragydere",
  "source.file.date.created": "Oprettet",
  "source.file.date.updated": "Sidste ændring",
  "tabs": "Faner",
  "toc": "Indholdsfortegnelse",
  "top": "Tilbage til start"
}[key] }}{% endmacro %}
